#include "StdAfx.h"
#include "Application.h"

CApplication * CApplication::instance = NULL;

CApplication::CApplication(void)
{
	CurrentScene = NULL;
}

CApplication::~CApplication(void)
{
}

void CApplication::Start(void) {
	InitWindow();
	InitGL();

	CurrentScene = new CScene();

	CurrentScene->Init();
	GameLoop();

	Cleanup();
}

void CApplication::InitWindow(void) {
	glfwInit();
	glfwOpenWindowHint(GLFW_FSAA_SAMPLES, 32);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MAJOR, OPENGL_MAJOR);
	glfwOpenWindowHint(GLFW_OPENGL_VERSION_MINOR, OPENGL_MINOR);
	glfwOpenWindow(640, 480, 8, 8, 8, 8, 24, 0, GLFW_WINDOW);
	glfwSetWindowPos(300, 200);
	glfwSetWindowTitle("VBOSandbox");
	glfwDisable(GLFW_AUTO_POLL_EVENTS);
	glfwSetTime(0.0);
}

void CApplication::InitGL(void) {
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	int w, h;
	glfwGetWindowSize(&w, &h);
	gluPerspective(60.0, (double) w / h, .1, 100.0);
	glViewport(0, 0, w, h);
	glClearColor(0, 0, 0, 0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glewInit();
}

void CApplication::ProcessEvents(void) {
	glfwPollEvents();
	CInput::GetInstance()->HandleInput();
}

void CApplication::Cleanup(void) {
	glfwTerminate();
}

double CApplication::GetTime(void) {
	return glfwGetTime();
}


void CApplication::Redraw() {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	CurrentScene->Redraw();

	glfwSwapBuffers();
}


void CApplication::Update() {
	ProcessEvents();
	CurrentScene->Update();
}


void CApplication::GameLoop() {
	
	double currentTime = GetTime();
	double nextUpdateTime = GetTime();

	int MaxFrameskip = 10;

	while (!CInput::GetInstance()->UserWantsToExit()) {
		currentTime = GetTime();

		int substeps = 0;
		while ((currentTime >= nextUpdateTime) && (substeps < MaxFrameskip)) {
			Update();
			++substeps;
			nextUpdateTime += UPDATE_INTERVAL;
		}

		Redraw();
	}

}

