#include "StdAfx.h"
#include "Scene.h"

#define N_SPHERES_PER_EDGE 5
#define N_SPHERES N_SPHERES_PER_EDGE * N_SPHERES_PER_EDGE * N_SPHERES_PER_EDGE

CScene::CScene(void)
{
	cameraAngle = 0.0;
	s = new CSphere(6);
	printf("Spheres: %d\n",  N_SPHERES);
	printf("Total vertices: %d\n", s->NFaces * 3 * N_SPHERES);
	CreateDisplayList();
	CreateVBO();
}


CScene::~CScene(void)
{
}

void CScene::Init(void) {
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);

	GLfloat lAmbient[] = {0.01f, 0.01f, 0.01f};
	GLfloat lDiffuse[] = {1.0f, 0.5f, 0.5f};
	GLfloat lSpecular[] = {0.0f, 0.0f, 0.0f};
	glLightfv(GL_LIGHT0, GL_AMBIENT, lAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lSpecular);

	GLfloat l1Ambient[] = {0.01f, 0.01f, 0.01f};
	GLfloat l1Diffuse[] = {0.5f, 0.5f, 1.0f};
	GLfloat l1Specular[] = {0.0f, 0.0f, 0.0f};
	glLightfv(GL_LIGHT1, GL_AMBIENT, l1Ambient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, l1Diffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, l1Specular);

	GLfloat mAmbient[] = {0.1f, 0.1f, 0.1f};
	GLfloat mDiffuse[] = {1.0f, 1.0f, 1.0f};
	GLfloat mSpecular[] = {1.0f, 1.0f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, mAmbient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mDiffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, mSpecular);
}

void CScene::Redraw(void) {
	DrawSphere();
}

void CScene::Update(void) {
	cameraAngle += .5;
}

void CScene::DrawSphere(void) {
	
	glTranslatef(0.0, 0.0, -15.0);
	glRotatef((GLfloat) cameraAngle, 0.5f, 1.0f, 0.25f);
	
	GLfloat pos[] = {0.0f, 0.0f, 10.0f, 2.0f};
	glLightfv(GL_LIGHT0, GL_POSITION, pos);
	GLfloat pos1[] = {0.0f, 0.0f, -10.0f, 2.0f};
	glLightfv(GL_LIGHT1, GL_POSITION, pos1);

	int NSpheres = N_SPHERES_PER_EDGE - 1;
	
	for (GLfloat x = -1.25f * NSpheres; x <= 1.25f * NSpheres; x += 2.5f) {
		for (GLfloat y = -1.25f * NSpheres; y <= 1.25f * NSpheres; y += 2.5f) {
			for (GLfloat z = -1.25f * NSpheres; z <= 1.25f * NSpheres; z += 2.5f) {
		
				glPushMatrix();
				glTranslatef(x, y, z);

				Draw_ImmediateMode();
				//Draw_DisplayList();
				//Draw_VBO();

				glPopMatrix();
	
			}
		}
	}

}

#pragma region Immediate mode

void CScene::Draw_ImmediateMode(void) {
	glBegin(GL_TRIANGLES);
		for (int i = 0; i < s->NFaces; ++i) {
			
			glNormal3f(s->Faces[i].p1.xn, s->Faces[i].p1.yn, s->Faces[i].p1.zn);
			glVertex3f(s->Faces[i].p1.x, s->Faces[i].p1.y, s->Faces[i].p1.z);

			glNormal3f(s->Faces[i].p2.xn, s->Faces[i].p2.yn, s->Faces[i].p2.zn);
			glVertex3f(s->Faces[i].p2.x, s->Faces[i].p2.y, s->Faces[i].p2.z);
			
			glNormal3f(s->Faces[i].p3.xn, s->Faces[i].p3.yn, s->Faces[i].p3.zn);
			glVertex3f(s->Faces[i].p3.x, s->Faces[i].p3.y, s->Faces[i].p3.z);

		}
	glEnd();
}

#pragma endregion

#pragma region Display list

void CScene::Draw_DisplayList(void) {
	glCallList(displayListId);
}

void CScene::CreateDisplayList(void) {
	
	displayListId = glGenLists(1);

	glNewList(displayListId, GL_COMPILE);

		glBegin(GL_TRIANGLES);
			for (int i = 0; i < s->NFaces; ++i) {
			
				glNormal3f(s->Faces[i].p1.xn, s->Faces[i].p1.yn, s->Faces[i].p1.zn);
				glVertex3f(s->Faces[i].p1.x, s->Faces[i].p1.y, s->Faces[i].p1.z);

				glNormal3f(s->Faces[i].p2.xn, s->Faces[i].p2.yn, s->Faces[i].p2.zn);
				glVertex3f(s->Faces[i].p2.x, s->Faces[i].p2.y, s->Faces[i].p2.z);
			
				glNormal3f(s->Faces[i].p3.xn, s->Faces[i].p3.yn, s->Faces[i].p3.zn);
				glVertex3f(s->Faces[i].p3.x, s->Faces[i].p3.y, s->Faces[i].p3.z);

			}
		glEnd();

	glEndList();

}

#pragma endregion

#pragma region VBO

#define VBO_STRIDE 6

void CScene::CreateVBO(void) {
	
	int NVertices = s->NFaces * 3;
	
	glGenBuffers(1, &vboId);
	glBindBuffer(GL_ARRAY_BUFFER, vboId);

	GLfloat * buf = new GLfloat[VBO_STRIDE * NVertices];
	for (int i = 0; i < s->NFaces; ++i) {

		buf[(3 * i + 0) * VBO_STRIDE + 0] = (GLfloat) s->Faces[i].p1.x;
		buf[(3 * i + 0) * VBO_STRIDE + 1] = (GLfloat) s->Faces[i].p1.y;
		buf[(3 * i + 0) * VBO_STRIDE + 2] = (GLfloat) s->Faces[i].p1.z;
		buf[(3 * i + 0) * VBO_STRIDE + 3] = (GLfloat) s->Faces[i].p1.xn;
		buf[(3 * i + 0) * VBO_STRIDE + 4] = (GLfloat) s->Faces[i].p1.yn;
		buf[(3 * i + 0) * VBO_STRIDE + 5] = (GLfloat) s->Faces[i].p1.zn;
		
		buf[(3 * i + 1) * VBO_STRIDE + 0] = (GLfloat) s->Faces[i].p2.x;
		buf[(3 * i + 1) * VBO_STRIDE + 1] = (GLfloat) s->Faces[i].p2.y;
		buf[(3 * i + 1) * VBO_STRIDE + 2] = (GLfloat) s->Faces[i].p2.z;
		buf[(3 * i + 1) * VBO_STRIDE + 3] = (GLfloat) s->Faces[i].p2.xn;
		buf[(3 * i + 1) * VBO_STRIDE + 4] = (GLfloat) s->Faces[i].p2.yn;
		buf[(3 * i + 1) * VBO_STRIDE + 5] = (GLfloat) s->Faces[i].p2.zn;

		buf[(3 * i + 2) * VBO_STRIDE + 0] = (GLfloat) s->Faces[i].p3.x;
		buf[(3 * i + 2) * VBO_STRIDE + 1] = (GLfloat) s->Faces[i].p3.y;
		buf[(3 * i + 2) * VBO_STRIDE + 2] = (GLfloat) s->Faces[i].p3.z;
		buf[(3 * i + 2) * VBO_STRIDE + 3] = (GLfloat) s->Faces[i].p3.xn;
		buf[(3 * i + 2) * VBO_STRIDE + 4] = (GLfloat) s->Faces[i].p3.yn;
		buf[(3 * i + 2) * VBO_STRIDE + 5] = (GLfloat) s->Faces[i].p3.zn;

	}
	
	glBufferData(GL_ARRAY_BUFFER, VBO_STRIDE * NVertices * sizeof(GLfloat), buf, GL_STATIC_DRAW);

	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_NORMAL_ARRAY);
	glVertexPointer(3, GL_FLOAT, VBO_STRIDE * sizeof(GLfloat), 0);
	glNormalPointer(GL_FLOAT, VBO_STRIDE * sizeof(GLfloat), (const GLvoid *) (3 * sizeof(GLfloat)));

	delete buf;
}

void CScene::Draw_VBO(void) {
	glBindBuffer(GL_ARRAY_BUFFER, vboId);
	glDrawArrays(GL_TRIANGLES, 0, s->NFaces * 3);
}

#pragma endregion
