#include "StdAfx.h"
#include "Sphere.h"


CSphere::CSphere(int iterations)
{
	Faces = new FACET3[(int) pow(4.0, iterations)];
	NFaces = CreateUnitSphere(iterations, Faces);
	for (int i = 0; i < NFaces; ++i) {
		Faces[i].p1.xn = Faces[i].p1.x;
		Faces[i].p1.yn = Faces[i].p1.y;
		Faces[i].p1.zn = Faces[i].p1.z;
		Faces[i].p2.xn = Faces[i].p2.x;
		Faces[i].p2.yn = Faces[i].p2.y;
		Faces[i].p2.zn = Faces[i].p2.z;
		Faces[i].p3.xn = Faces[i].p3.x;
		Faces[i].p3.yn = Faces[i].p3.y;
		Faces[i].p3.zn = Faces[i].p3.z;
	}
	printf("Generated a sphere: %d vertices\n", NFaces * 3);
}


CSphere::~CSphere(void)
{
}

/*
	Create a unit sphere centered at the origin
	This code illustrates the concept rather than implements it efficiently
	The maximum number of Faces produced is iterations * 3 (this is returned)
*/
int CSphere::CreateUnitSphere(int iterations, FACET3 * Faces)
{
	int i,j,n,nstart;
	XYZ p1 = {1.0,1.0,1.0},  p2 = {-1.0,-1.0,1.0};
	XYZ p3 = {1.0,-1.0,-1.0}, p4 = {-1.0,1.0,-1.0}; 
	Normalise(&p1);
	Normalise(&p2);
	Normalise(&p3);
	Normalise(&p4);

	Faces[0].p1 = p1; Faces[0].p2 = p2; Faces[0].p3 = p3;
	Faces[1].p1 = p2; Faces[1].p2 = p1; Faces[1].p3 = p4;
	Faces[2].p1 = p2; Faces[2].p2 = p4; Faces[2].p3 = p3;
	Faces[3].p1 = p1; Faces[3].p2 = p3; Faces[3].p3 = p4;

	n = 4;
	
	for (i=1;i<iterations;i++) {
		nstart = n;

		for (j=0;j<nstart;j++) {
			
			/* Create initially copies for the new Faces */
			Faces[n  ] = Faces[j];
			Faces[n+1] = Faces[j];
			Faces[n+2] = Faces[j];

			/* Calculate the midpoints */
			p1 = MidPoint(Faces[j].p1,Faces[j].p2);
			p2 = MidPoint(Faces[j].p2,Faces[j].p3);
			p3 = MidPoint(Faces[j].p3,Faces[j].p1);

			/* Replace the current facet */
			Faces[j].p2 = p1;
			Faces[j].p3 = p3;

			/* Create the changed vertices in the new Faces */
			Faces[n  ].p1 = p1;
			Faces[n  ].p3 = p2;
			Faces[n+1].p1 = p3;
			Faces[n+1].p2 = p2;
			Faces[n+2].p1 = p1;
			Faces[n+2].p2 = p2;
			Faces[n+2].p3 = p3;
			n += 3;
		}
	}

	for (j=0;j<n;j++) {
		Normalise(&Faces[j].p1);
		Normalise(&Faces[j].p2);
		Normalise(&Faces[j].p3);
	}

	return(n);
}

/*
   Return the midpoint between two vectors
*/
XYZ CSphere::MidPoint(XYZ p1, XYZ p2)
{
   XYZ p;

   p.x = (p1.x + p2.x) / 2;
   p.y = (p1.y + p2.y) / 2;
   p.z = (p1.z + p2.z) / 2;

   return(p);
}

/*
   Normalise a vector
*/
void CSphere::Normalise(XYZ *p)
{
   double length;

   length = sqrt(p->x * p->x + p->y * p->y + p->z * p->z);
   if (length != 0) {
      p->x /= length;
      p->y /= length;
      p->z /= length;
   } else {
      p->x = 0;
      p->y = 0;
      p->z = 0;
   }
}
